
package gus05.manager.gus.endorf.layer2.servicewrap;

import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.JComponent;

import gus05.framework.core.Service;
import gus05.framework.features.*;
import gus05.manager.gus.endorf.layer2.EndorfSystem;


public abstract class AbstractServiceWrap implements Service {

    
    protected Object target;
    
    protected Execute execute;
    protected Filter filter;
    protected Function function;
    protected Give give;
    protected Graphic graphic;
    protected Provide provide;
    protected Register register;
    protected Retrieve retrieve;
    protected Support support;
    protected Transform transform;
    protected Runnable runnable;
    
    
    public AbstractServiceWrap(Object target)
    {
        if(target==null)return;
        this.target = target;
        
        if(target instanceof Execute)		execute = (Execute)target;
        if(target instanceof Filter)		filter = (Filter)target;
        if(target instanceof Function)		function = (Function)target;
        if(target instanceof Give)			give = (Give)target;
        if(target instanceof Graphic)		graphic = (Graphic)target;
        if(target instanceof Provide)		provide = (Provide)target;
        if(target instanceof Register)		register = (Register)target;
        if(target instanceof Retrieve)		retrieve = (Retrieve)target;
        if(target instanceof Execute)		execute = (Execute)target;
        if(target instanceof Support)		support = (Support)target;
        if(target instanceof Transform)		transform = (Transform)target;
        if(target instanceof Runnable)		runnable = (Runnable)target;
    }
    
    
    protected abstract void featureStart(String feature, Object input);
    protected abstract void featureEnd(String feature, Object output);
    
    protected abstract void notAvailable1(String feature) throws Exception;
    protected abstract void notAvailable2(String feature);
    
    
    
    public void execute() throws Exception
    {
        if(execute==null)
            notAvailable1(EndorfSystem.FEATURENAME_EXECUTE);
        
        featureStart(EndorfSystem.FEATURENAME_EXECUTE,null);
        try{execute.execute();}
        catch(Exception e)
        {
        	EndorfSystem.printDebug(execute,"execute()",e.toString());
        	throw e;
        }
        featureEnd(EndorfSystem.FEATURENAME_EXECUTE,null);
    }
    

    public boolean filter(Object obj) throws Exception
    {
        if(filter==null)
            notAvailable1(EndorfSystem.FEATURENAME_FILTER);
        
        featureStart(EndorfSystem.FEATURENAME_FILTER,obj);
        boolean result = false;
        try{result = filter.filter(obj);}
        catch(Exception e)
        {
        	EndorfSystem.printDebug(filter,"filter(Object)",e.toString());
        	throw e;
        }
        featureEnd(EndorfSystem.FEATURENAME_FILTER,Boolean.valueOf(result));
        
        return result;
    }

    
    public double function(double value) throws Exception
    {
        if(function==null)
            notAvailable1(EndorfSystem.FEATURENAME_FUNCTION);
        
        featureStart(EndorfSystem.FEATURENAME_FUNCTION,Double.valueOf(value));
        double result = 0;
        try{result = function.function(value);}
        catch(Exception e)
        {
        	EndorfSystem.printDebug(function,"function(double)",e.toString());
        	throw e;
        }
        featureEnd(EndorfSystem.FEATURENAME_FUNCTION,Double.valueOf(result));
        
        return result;
    }
    

    public void give(Object obj) throws Exception
    {
        if(give==null)
            notAvailable1(EndorfSystem.FEATURENAME_GIVE);
        
        featureStart(EndorfSystem.FEATURENAME_GIVE,obj);
        try{give.give(obj);}
        catch(Exception e)
        {
        	EndorfSystem.printDebug(give,"give(Object)",e.toString());
        	throw e;
        }
        featureEnd(EndorfSystem.FEATURENAME_GIVE,null);
    }

    
    public Object provide() throws Exception
    {
        if(provide==null)
	        notAvailable1(EndorfSystem.FEATURENAME_PROVIDE);
        
        featureStart(EndorfSystem.FEATURENAME_PROVIDE,null);
        Object result = null;
        try{result = provide.provide();}
        catch(Exception e)
        {
        	EndorfSystem.printDebug(provide,"provide()",e.toString());
        	throw e;
        }
        featureEnd(EndorfSystem.FEATURENAME_PROVIDE,result);
        
        return result;
    }

    
    public void register(String key, Object obj) throws Exception
    {
        if(register==null)
            notAvailable1(EndorfSystem.FEATURENAME_REGISTER);
        
        featureStart(EndorfSystem.FEATURENAME_REGISTER,new Object[]{key,obj});
        try{register.register(key,obj);}
        catch(Exception e)
        {
        	EndorfSystem.printDebug(register,"register(String,Object)",e.toString());
        	throw e;
        }
        featureEnd(EndorfSystem.FEATURENAME_REGISTER,null);
    }

    
    public Object retrieve(String key) throws Exception
    {
        if(retrieve==null)
            notAvailable1(EndorfSystem.FEATURENAME_RETRIEVE);
        
        featureStart(EndorfSystem.FEATURENAME_RETRIEVE,key);
        Object result = null;
        try{result = retrieve.retrieve(key);}
        catch(Exception e)
        {
        	EndorfSystem.printDebug(retrieve,"retrieve(String)",e.toString());
        	throw e;
        }
        featureEnd(EndorfSystem.FEATURENAME_RETRIEVE,result);
        
        return result;
    }
    
    
    public Object transform(Object obj) throws Exception
    {
        if(transform==null)
            notAvailable1(EndorfSystem.FEATURENAME_TRANSFORM);
        
        featureStart(EndorfSystem.FEATURENAME_TRANSFORM,obj);
        Object result = null;
        try{result = transform.transform(obj);}
        catch(Exception e)
        {
        	EndorfSystem.printDebug(transform,"transform(Object)",e.toString());
        	throw e;
        }
        featureEnd(EndorfSystem.FEATURENAME_TRANSFORM,result);
        
        return result;
    }
    
    
    
    public JComponent gui()
    {
        if(graphic==null)
        {notAvailable2(EndorfSystem.FEATURENAME_GRAPHIC);return null;}
        
        featureStart(EndorfSystem.FEATURENAME_GRAPHIC,null);
        JComponent result = graphic.gui();
        featureEnd(EndorfSystem.FEATURENAME_GRAPHIC,result);
        
        return result;
    }
    
    
    public void addActionListener(ActionListener listener)
    {
        if(support==null)
        {notAvailable2(EndorfSystem.FEATURENAME_SUPPORT_ADD);return;}
        
        featureStart(EndorfSystem.FEATURENAME_SUPPORT_ADD,listener);
        support.addActionListener(listener);
        featureEnd(EndorfSystem.FEATURENAME_SUPPORT_ADD,null);
    }

    
    public void removeActionListener(ActionListener listener)
    {
        if(support==null)
        {notAvailable2(EndorfSystem.FEATURENAME_SUPPORT_REMOVE);return;}
        
        featureStart(EndorfSystem.FEATURENAME_SUPPORT_REMOVE,listener);
        support.removeActionListener(listener);
        featureEnd(EndorfSystem.FEATURENAME_SUPPORT_REMOVE,null);
    }

    
    public List listeners()
    {
        if(support==null)
        {notAvailable2(EndorfSystem.FEATURENAME_SUPPORT_LIST);return null;}
        
        featureStart(EndorfSystem.FEATURENAME_SUPPORT_LIST,null);
        List result = support.listeners();
        featureEnd(EndorfSystem.FEATURENAME_SUPPORT_LIST,result);
        return result;
    }

    
    public void run()
    {
        if(runnable==null)
        {notAvailable2(EndorfSystem.FEATURENAME_RUN);return;}
        
        featureStart(EndorfSystem.FEATURENAME_RUN,null);
        runnable.run();
        featureEnd(EndorfSystem.FEATURENAME_RUN,null);
    }

}
