package gus05.manager.gus.endorf.layer3.loading;

import gus05.manager.gus.endorf.layer3.system1.System1;
import gus05.manager.gus.endorf.layer3.tools.Tools_file;
import gus05.manager.gus.endorf.layer3.tools.Tools_fileFilter;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.ImageObserver;
import java.io.File;
import java.net.URL;
import java.util.Iterator;
import java.util.Properties;
import java.util.Set;

import javax.accessibility.AccessibleContext;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.filechooser.FileSystemView;


public class Loader_icons extends Loader {

    
    public Loader_icons()
    {super("icons");}
    
    
    
    
    
    
    protected void insideLoad()
    {
        try
        {
            Set set = insideDir("icon");
            if(set==null) return;

            Iterator it = set.iterator();
            while(it.hasNext())
            {
                String name = (String)it.next();
                URL url = insideURL("icon/"+name);
                //debug("insideLoad()","inside url: "+url);
                loadIcon(name,url);
            }
            debug("insideLoad()","load "+set.size()+" icons from inside");
        }
        catch(Exception e)
        {System1.handleError(this,"insideLoad()",e);}
    }
    
    
    
    
    protected void outsideLoad()
    {
        loadFromDir();
        loadFromFile();
    }
    
    
    
    
    
    private void loadFromDir()
    {
        try
        {           
            File iconDir = outsideDir(System1.PATH_ICONDIR);
            if(iconDir==null)return;
            
            File[] gifs = iconDir.listFiles(Tools_fileFilter.gifFileFilter);
            debug("loadFromDir()","load "+gifs.length+" gif icons from: "+iconDir.getAbsolutePath());
            for(int i=0;i<gifs.length;i++) loadFile(gifs[i]);
            
            File[] pngs = iconDir.listFiles(Tools_fileFilter.pngFileFilter);
            debug("loadFromDir()","load "+pngs.length+" png icons from: "+iconDir.getAbsolutePath());
            for(int i=0;i<pngs.length;i++) loadFile(pngs[i]);
            
            
            File[] d = iconDir.listFiles(Tools_fileFilter.dirFileFilter);
            for(int i=0;i<d.length;i++)
            {
                String base = d[i].getName();
                
                File[] gifs0 = d[i].listFiles(Tools_fileFilter.gifFileFilter);
                debug("loadFromDir()","load "+gifs0.length+" gif icons from: "+d[i].getAbsolutePath());
                for(int j=0;j<gifs0.length;j++) loadFile(base,gifs0[j]);
                
                File[] pngs0 = d[i].listFiles(Tools_fileFilter.pngFileFilter);
                debug("loadFromDir()","load "+pngs0.length+" png icons from: "+d[i].getAbsolutePath());
                for(int j=0;j<pngs0.length;j++) loadFile(base,pngs0[j]);
            }
        }
        catch(Exception e)
        {System1.handleError(this,"loadFromDir()",e);}
    }
    
    
    
    
    
    
    private void loadFromFile()
    {
        try
        {           
            File f = outsideFile(System1.PATH_ICONFILESYSTEM);
            if(f==null)return;
            
            Properties p = Tools_file.prop(f);
            
            Iterator it = p.keySet().iterator();
            while(it.hasNext())
            {
                String iconID = (String) it.next();
                String path = p.getProperty(iconID);
                File file = System1.searchFile(path);
                if(file.exists())
                {
                    Icon icon = FileSystemView.getFileSystemView().getSystemIcon(file);
                    putIcon(iconID,icon);
                }
                else warning("loadFromFile()","file "+file.getAbsolutePath()+" not found");
            }
            
            debug("loadFromFile()","load "+p.size()+" icons from: "+f.getAbsolutePath());
        }
        catch(Exception e)
        {System1.handleError(this,"loadFromFile()",e);}
    }
    
    
    
    
    
    
    
    
    
    private void loadFile(File f)
    {
        try{loadIcon(f.getName(),f.toURI().toURL());}
        catch(Exception e)
        {System1.handleError(this,"loadGifFile(File)",e);}
    }
    
    
    
    
    private void loadFile(String base, File f)
    {
        try{loadIcon(base+"@"+f.getName(),f.toURI().toURL());}
        catch(Exception e)
        {System1.handleError(this,"loadGifFile(String,File)",e);}
    }
    
    
    
    
    
    
    
    private void loadIcon(String name, URL url) throws Exception
    {
        if(!name.endsWith(".gif") && !name.endsWith(".png"))
        	throw new Exception("wrong icon name: "+name);
        
        String key = name.substring(0,name.length()-4);
        ImageIcon0 icon = new ImageIcon0(url);
        putIcon(key,icon);
    }
    
    
    private void putIcon(String key, Icon icon)
    {
        if(System1.iconMap.containsKey(key))
            warning("putIcon(String,URL)","iconID: "+key+" is loaded many times");
        System1.iconMap.put(key,icon);
        increase();
    }
    
    
    
    
    
    
    
    
    
    
    
    
    
    private class ImageIcon0 extends ImageIcon
    {
    	private URL url;
    	private ImageIcon icon;
    	
    	public ImageIcon0(URL url)
    	{this.url = url;}
    	
    	private void init()
    	{if(icon==null) icon = new ImageIcon(url);}
    	
    	public AccessibleContext getAccessibleContext()
    	{init();return icon.getAccessibleContext();}
         
    	public String getDescription()
    	{init();return icon.getDescription();}
         
    	public int getIconHeight()
    	{init();return icon.getIconHeight();}
         
    	public int getIconWidth()
    	{init();return icon.getIconWidth();}

    	public Image getImage()
    	{init();return icon.getImage();}

    	public int getImageLoadStatus()
    	{init();return icon.getImageLoadStatus();}

    	public ImageObserver getImageObserver()
    	{init();return icon.getImageObserver();}
    	
    	public void paintIcon(Component c, Graphics g, int x, int y)
    	{init();icon.paintIcon(c,g,x,y);}
    	
     	public void setDescription(String description)
     	{init();icon.setDescription(description);}
     	
     	public void setImage(Image image)
     	{init();icon.setImage(image);}
     	
     	public void setImageObserver(ImageObserver observer)
     	{init();icon.setImageObserver(observer);}
     	
     	public String toString()
     	{init();return icon.toString();}
    }
}
