package gus05.manager.gus.endorf.layer3.tools;

import gus05.manager.gus.endorf.layer3.system1.System1;

import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.spec.DSAPrivateKeySpec;
import java.security.spec.DSAPublicKeySpec;
import java.util.Properties;




public class Tools_publicKey {

    
    
    
    private static KeyFactory kf;
    private static Signature si;
    
    
    
    
    private static void init() throws Exception
    {
        if(kf==null) kf = KeyFactory.getInstance(SECURITY.SECURITY_ALGO_DSA);
        if(si==null) si = Signature.getInstance(SECURITY.SECURITY_ALGO_DSA);
    }
    
    
    
    
    
    
    
    
    
    
    
    
    public final static PublicKey buildPublicKey(Properties prop) throws Exception
    {
        init();
        
        String y = findValue_public(prop,SECURITY.PUBLICKEY_Y0);
        String p = findValue_public(prop,SECURITY.PUBLICKEY_P0);
        String q = findValue_public(prop,SECURITY.PUBLICKEY_Q0);
        String g = findValue_public(prop,SECURITY.PUBLICKEY_G0);
        String owner = findValue_public(prop,SECURITY.PUBLICKEY_OWNER);
        String sign = findValue_public(prop,SECURITY.PUBLICKEY_SIGN);
        
        if(owner.equals("augustin.delale.06081977"))
        {
            String y_ = "4517575272117675090020619265258517436262221026902616530261677237959284587872138777828846836705449477570567249122240682050678251594040682645116912167989747";
            String p_ = "13232376895198612407547930718267435757728527029623408872245156039757713029036368719146452186041204237350521785240337048752071462798273003935646236777459223";
            String q_ = "857393771208094202104259627990318636601332086981";
            String g_ = "5421644057436475141609648488325705128047428394380474376834667300766108262613900542681289080713724597310673074119355136085795982097390670890367185141189796";
            
            if(!g.equals(g_)) corruptedPublicKey(owner);
            if(!q.equals(q_)) corruptedPublicKey(owner);
            if(!p.equals(p_)) corruptedPublicKey(owner);
            if(!y.equals(y_)) corruptedPublicKey(owner);
        }
        
        BigInteger G = new BigInteger(g);
        BigInteger P = new BigInteger(p);
        BigInteger Q = new BigInteger(q);
        BigInteger Y = new BigInteger(y);
        
        DSAPublicKeySpec keySpec = new DSAPublicKeySpec(Y,P,Q,G);
        PublicKey publicKey = kf.generatePublic(keySpec);
        
        si.initVerify(publicKey);
        si.update(owner.getBytes());
        byte[] signData = Tools_base64.decodeBase64(sign);
        boolean valid = si.verify(signData);
        if(!valid) corruptedPublicKey(owner);
        
        return publicKey;
    }
    
    private static String findValue_public(Properties prop, String key) throws Exception
    {
    	if(!prop.containsKey(key)) throw new Exception("Invalid public key properties: "+key+" is missing");
    	return prop.getProperty(key);
    }
    
    
    
    
    
    
    
    
    
    public final static PrivateKey buildPrivateKey(Properties prop) throws Exception
    {
        init();
        
        String g = findValue_private(prop,SECURITY.PRIVATEKEY_G1);
        String p = findValue_private(prop,SECURITY.PRIVATEKEY_P1);
        String q = findValue_private(prop,SECURITY.PRIVATEKEY_Q1);
        String x = findValue_private(prop,SECURITY.PRIVATEKEY_X1);
        String owner = findValue_private(prop,SECURITY.PRIVATEKEY_OWNER);
        
        BigInteger G = new BigInteger(g);
        BigInteger P = new BigInteger(p);
        BigInteger Q = new BigInteger(q);
        BigInteger X = new BigInteger(x);
        
        DSAPrivateKeySpec keySpec = new DSAPrivateKeySpec(X,P,Q,G);
        return kf.generatePrivate(keySpec);
    }
    
    private static String findValue_private(Properties prop, String key) throws Exception
    {
    	if(!prop.containsKey(key)) throw new Exception("Invalid private key properties: "+key+" is missing");
    	return prop.getProperty(key);
    }
    
    
    
    
    
    public static Properties publicProp(PublicKey publicKey, String owner, byte[] signData) throws Exception
    {
        init();
        
        DSAPublicKeySpec spec = (DSAPublicKeySpec) kf.getKeySpec(publicKey,DSAPublicKeySpec.class);
        String sign = Tools_base64.encodeBase64(signData);
        
        Properties prop = new Properties();
        prop.setProperty(SECURITY.PUBLICKEY_G0,spec.getG().toString());
        prop.setProperty(SECURITY.PUBLICKEY_P0,spec.getP().toString());
        prop.setProperty(SECURITY.PUBLICKEY_Q0,spec.getQ().toString());
        prop.setProperty(SECURITY.PUBLICKEY_Y0,spec.getY().toString());
        prop.setProperty(SECURITY.PUBLICKEY_OWNER,owner);
        prop.setProperty(SECURITY.PUBLICKEY_SIGN,sign);
        return prop;
    }
    
    
    
    
    
    
    
    
    public static Properties privateProp(PrivateKey privateKey, String owner, byte[] signData) throws Exception
    {
        init();
        
        DSAPrivateKeySpec spec = (DSAPrivateKeySpec) kf.getKeySpec(privateKey,DSAPrivateKeySpec.class);
        String sign = Tools_base64.encodeBase64(signData);
        
        Properties prop = new Properties();
        prop.setProperty(SECURITY.PRIVATEKEY_G1,spec.getG().toString());
        prop.setProperty(SECURITY.PRIVATEKEY_P1,spec.getP().toString());
        prop.setProperty(SECURITY.PRIVATEKEY_Q1,spec.getQ().toString());
        prop.setProperty(SECURITY.PRIVATEKEY_X1,spec.getX().toString());
        prop.setProperty(SECURITY.PRIVATEKEY_OWNER,owner);
        prop.setProperty(SECURITY.PRIVATEKEY_SIGN,sign);
        return prop;
    }
    
    
    
    
    
    private static void corruptedPublicKey(String owner)
    {System1.FATAL_ERROR(Tools_publicKey.class,"corruptedPublicKey(String)","corrupted public key for owner "+owner);}
    
    
}
